@@ -100,16 +100,6 @@ extern uint16_t EDNS_ADVERTISED_SIZE;
  */
 int str_is_ip6(const char* str);
 
-/**
- * Write (blocking) to a nonblocking socket.
- * @param s: fd. Is set to be nonblocking at exit.
- * @param buf: data buffer.
- * @param size: length of data to send.
- * @return: 0 on error. errno is set.
- */
-int
-write_socket(int s, const void *buf, size_t size);
-
 /**
  * Set fd nonblocking.
  * @param s: file descriptor.
@@ -233,7 +223,7 @@ int sockaddr_cmp_addr(struct sockaddr_storage* addr1, socklen_t len1,
  * Checkout address family.
  * @param addr: the sockaddr to examine.
  * @param len: the length of addr.
- * return: true if sockaddr is ip6.
+ * @return: true if sockaddr is ip6.
  */
 int addr_is_ip6(struct sockaddr_storage* addr, socklen_t len);
 
@@ -272,7 +262,7 @@ void addr_to_str(struct sockaddr_storage* addr, socklen_t addrlen,
 	char* buf, size_t len);
 
 /**
- * See if sockaddr is an ipv6 mapped ipv4 address, ::ffff:0.0.0.0
+ * See if sockaddr is an ipv6 mapped ipv4 address, "::ffff:0.0.0.0"
  * @param addr: address
  * @param addrlen: length of address
  * @return true if so
@@ -287,6 +277,14 @@ int addr_is_ip4mapped(struct sockaddr_storage* addr, socklen_t addrlen);
  */
 int addr_is_broadcast(struct sockaddr_storage* addr, socklen_t addrlen);
 
+/**
+ * See if sockaddr is 0.0.0.0 or ::0.
+ * @param addr: address
+ * @param addrlen: length of address
+ * @return true if so
+ */
+int addr_is_any(struct sockaddr_storage* addr, socklen_t addrlen);
+
 /**
  * Insert new socket list item. If fails logs error.
  * @param list: pointer to pointer to first item.
@@ -325,4 +323,44 @@ int sock_list_find(struct sock_list* list, struct sockaddr_storage* addr,
 void sock_list_merge(struct sock_list** list, struct regional* region,
 	struct sock_list* add);
 
+/**
+ * Log libcrypto error with descriptive string. Calls log_err().
+ * @param str: what failed.
+ */
+void log_crypto_err(const char* str);
+
+/** 
+ * create SSL listen context
+ * @param key: private key file.
+ * @param pem: public key cert.
+ * @param verifypem: if nonNULL, verifylocation file.
+ * return SSL_CTX* or NULL on failure (logged).
+ */
+void* listen_sslctx_create(char* key, char* pem, char* verifypem);
+
+/**
+ * create SSL connect context
+ * @param key: if nonNULL (also pem nonNULL), the client private key.
+ * @param pem: client public key (or NULL if key is NULL).
+ * @param verifypem: if nonNULL used for verifylocation file.
+ * @return SSL_CTX* or NULL on failure (logged).
+ */
+void* connect_sslctx_create(char* key, char* pem, char* verifypem);
+
+/**
+ * accept a new fd and wrap it in a BIO in SSL
+ * @param sslctx: the SSL_CTX to use (from listen_sslctx_create()).
+ * @param fd: from accept, nonblocking.
+ * @return SSL or NULL on alloc failure.
+ */
+void* incoming_ssl_fd(void* sslctx, int fd);
+
+/**
+ * connect a new fd and wrap it in a BIO in SSL
+ * @param sslctx: the SSL_CTX to use (from connect_sslctx_create())
+ * @param fd: from connect.
+ * @return SSL or NULL on alloc failure
+ */
+void* outgoing_ssl_fd(void* sslctx, int fd);
+
 #endif /* NET_HELP_H */
