@@ -643,7 +643,9 @@ nsec3_hash_name(rbtree_t* table, struct regional* region, ldns_buffer* buf,
 {
 	struct nsec3_cached_hash* c;
 	struct nsec3_cached_hash looki;
+#ifdef UNBOUND_DEBUG
 	rbnode_t* n;
+#endif
 	int r;
 	looki.node.key = &looki;
 	looki.nsec3 = nsec3;
@@ -670,7 +672,10 @@ nsec3_hash_name(rbtree_t* table, struct regional* region, ldns_buffer* buf,
 	r = nsec3_calc_b32(region, buf, c);
 	if(r != 1)
 		return r;
-	n = rbtree_insert(table, &c->node);
+#ifdef UNBOUND_DEBUG
+	n =
+#endif
+	rbtree_insert(table, &c->node);
 	log_assert(n); /* cannot be duplicate, just did lookup */
 	*hash = c;
 	return 1;
@@ -1191,8 +1196,7 @@ nsec3_do_prove_nodata(struct module_env* env, struct nsec3_filter* flt,
 			return sec_status_bogus;
 		}
 		/* everything is peachy keen, except for optout spans */
-		log_assert(ce.nc_rrset);
-		if(nsec3_has_optout(ce.nc_rrset, ce.nc_rr)) {
+		if(ce.nc_rrset && nsec3_has_optout(ce.nc_rrset, ce.nc_rr)) {
 			verbose(VERB_ALGO, "nsec3 nodata proof: matching "
 				"wildcard is in optout range, insecure");
 			return sec_status_insecure;
@@ -1204,6 +1208,10 @@ nsec3_do_prove_nodata(struct module_env* env, struct nsec3_filter* flt,
 	/* Due to forwarders, cnames, and other collating effects, we
 	 * can see the ordinary unsigned data from a zone beneath an
 	 * insecure delegation under an optout here */
+	if(!ce.nc_rrset) {
+		verbose(VERB_ALGO, "nsec3 nodata proof: no next closer nsec3");
+		return sec_status_bogus;
+	}
 
 	/* We need to make sure that the covering NSEC3 is opt-out. */
 	log_assert(ce.nc_rrset);
@@ -1378,6 +1386,13 @@ nsec3_prove_nods(struct module_env* env, struct val_env* ve,
 		return sec_status_bogus;
 	}
 
+	/* robust extra check */
+	if(!ce.nc_rrset) {
+		verbose(VERB_ALGO, "nsec3 nods proof: no next closer nsec3");
+		*reason = "no NSEC3 next closer";
+		return sec_status_bogus;
+	}
+
 	/* we had the closest encloser proof, then we need to check that the
 	 * covering NSEC3 was opt-out -- the proveClosestEncloser step already
 	 * checked to see if the closest encloser was a delegation or DNAME.
